@@ -9,7 +9,7 @@
 # option) any later version.  See http://www.gnu.org/copyleft/gpl.html for
 # the full text of the license.
 
-import subprocess, tempfile, os.path, re, pwd, grp, os, time
+import subprocess, tempfile, os.path, re, pwd, grp, os, time, io
 import fnmatch, glob, traceback, errno, sys, atexit, locale, imp
 
 import xml.dom, xml.dom.minidom
@@ -64,51 +64,72 @@ def _transitive_dependencies(package, depends_set):
             _transitive_dependencies(d, depends_set)
 
 
-def _read_file(path):
+def _read_proc_link(path, pid=None, dir_fd=None):
+    '''Use readlink() to resolve link.
+
+    Return a string representing the path to which the symbolic link points.
+    '''
+    if not _python2 and dir_fd is not None:
+        return os.readlink(path, dir_fd=dir_fd)
+
+    return os.readlink("/proc/%s/%s" % (pid, path))
+
+
+def _read_proc_file(path, pid=None, dir_fd=None):
     '''Read file content.
 
     Return its content, or return a textual error if it failed.
     '''
     try:
-        with open(path, 'rb') as fd:
+        if not _python2 and dir_fd is not None:
+            proc_file = os.open(path, os.O_RDONLY | os.O_CLOEXEC, dir_fd=dir_fd)
+        else:
+            proc_file = "/proc/%s/%s" % (pid, path)
+
+        with io.open(proc_file, 'rb') as fd:
             return fd.read().strip().decode('UTF-8', errors='replace')
     except (OSError, IOError) as e:
         return 'Error: ' + str(e)
 
 
-def _read_maps(pid):
+def _read_maps(proc_pid_fd):
     '''Read /proc/pid/maps.
 
     Since /proc/$pid/maps may become unreadable unless we are ptracing the
     process, detect this, and attempt to attach/detach.
     '''
     maps = 'Error: unable to read /proc maps file'
+
+    if _python2:
+        return 'Error: python2 does not provide a secure way to read /proc maps file'
+
     try:
-        with open('/proc/%d/maps' % pid) as fd:
+        with open('maps', opener=lambda path, mode: os.open(path, mode, dir_fd=proc_pid_fd)) as fd:
             maps = fd.read().strip()
     except (OSError, IOError) as e:
         return 'Error: ' + str(e)
     return maps
 
 
-def _command_output(command, input=None, stderr=subprocess.STDOUT):
+def _command_output(command, input=None, env=None):
     '''Run command and capture its output.
 
     Try to execute given command (argv list) and return its stdout, or return
     a textual error if it failed.
     '''
-    sp = subprocess.Popen(command, stdout=subprocess.PIPE, stderr=stderr)
+    sp = subprocess.Popen(command, stdout=subprocess.PIPE,
+                          stderr=subprocess.STDOUT, env=env)
 
-    (out, err) = sp.communicate(input)
+    out = sp.communicate(input)[0]
     if sp.returncode == 0:
         return out
     else:
-        if err:
-            err = err.decode('UTF-8', errors='replace')
+        if out:
+            out = out.decode('UTF-8', errors='replace')
         else:
-            err = ''
+            out = ''
         raise OSError('Error: command %s failed with exit code %i: %s' % (
-            str(command), sp.returncode, err))
+            str(command), sp.returncode, out))
 
 
 def _check_bug_pattern(report, pattern):
@@ -147,7 +168,7 @@ def _check_bug_pattern(report, pattern):
                         regexp = regexp.encode('UTF-8')
                 try:
                     re_c = re.compile(regexp)
-                except:
+                except Exception:
                     continue
                 if not re_c.search(v):
                     return None
@@ -204,7 +225,7 @@ def _run_hook(report, ui, hook):
                 raise
     except StopIteration:
         return True
-    except:
+    except Exception as e:
         hookname = os.path.splitext(os.path.basename(hook))[0].replace('-', '_')
         report['HookError_' + hookname] = traceback.format_exc()
         apport.error('hook %s crashed:', hook)
@@ -212,6 +233,23 @@ def _run_hook(report, ui, hook):
 
     return False
 
+
+def _which_extrapath(command, extra_path):
+    '''Return path of command, preferring extra_path'''
+
+    if extra_path:
+        env = os.environ.copy()
+        parts = env.get('PATH', '').split(os.pathsep)
+        parts[0:0] = extra_path.split(os.pathsep)
+        env['PATH'] = os.pathsep.join(parts)
+    else:
+        env = None
+    try:
+        return subprocess.check_output(['which', command], env=env).decode().strip()
+    except subprocess.CalledProcessError:
+        return None
+
+
 #
 # Report class
 #
@@ -380,9 +418,9 @@ class Report(problem_report.ProblemReport):
 
         # first, determine process name
         name = None
-        for l in self['ProcStatus'].splitlines():
+        for line in self['ProcStatus'].splitlines():
             try:
-                (k, v) = l.split('\t', 1)
+                (k, v) = line.split('\t', 1)
             except ValueError:
                 continue
             if k == 'Name:':
@@ -486,7 +524,7 @@ class Report(problem_report.ProblemReport):
             path = path[:-1]
         return path
 
-    def add_proc_info(self, pid=None, extraenv=[]):
+    def add_proc_info(self, pid=None, proc_pid_fd=None, extraenv=[]):
         '''Add /proc/pid information.
 
         If neither pid nor self.pid are given, it defaults to the process'
@@ -510,27 +548,39 @@ class Report(problem_report.ProblemReport):
         - _LogindSession: logind cgroup path, if present (Used for filtering
           out crashes that happened in a session that is not running any more)
         '''
-        if not pid:
-            pid = self.pid or os.getpid()
-        if not self.pid:
-            self.pid = int(pid)
-        pid = str(pid)
+        if not proc_pid_fd:
+            if not pid:
+                pid = self.pid or os.getpid()
+            if not self.pid:
+                self.pid = int(pid)
+            pid = str(pid)
+            if not _python2:
+                try:
+                    proc_pid_fd = os.open('/proc/%s' % pid, os.O_RDONLY | os.O_PATH | os.O_DIRECTORY)
+                except OSError as e:
+                    if e.errno in (errno.EPERM, errno.EACCES):
+                        raise ValueError('not accessible')
+                    if e.errno == errno.ENOENT:
+                        raise ValueError('invalid process')
+                    else:
+                        raise
 
         try:
-            self['ProcCwd'] = os.readlink('/proc/' + pid + '/cwd')
+            self['ProcCwd'] = _read_proc_link('cwd', pid, proc_pid_fd)
         except OSError:
             pass
-        self.add_proc_environ(pid, extraenv)
-        self['ProcStatus'] = _read_file('/proc/' + pid + '/status')
-        self['ProcCmdline'] = _read_file('/proc/' + pid + '/cmdline').rstrip('\0')
-        self['ProcMaps'] = _read_maps(int(pid))
-        try:
-            self['ExecutablePath'] = os.readlink('/proc/' + pid + '/exe')
-        except OSError as e:
-            if e.errno == errno.ENOENT:
-                raise ValueError('invalid process')
-            else:
-                raise
+        self.add_proc_environ(pid=pid, proc_pid_fd=proc_pid_fd, extraenv=extraenv)
+        self['ProcStatus'] = _read_proc_file('status', pid, proc_pid_fd)
+        self['ProcCmdline'] = _read_proc_file('cmdline', pid, proc_pid_fd).rstrip('\0')
+        self['ProcMaps'] = _read_maps(proc_pid_fd)
+        if 'ExecutablePath' not in self:
+            try:
+                self['ExecutablePath'] = _read_proc_link('exe', pid, proc_pid_fd)
+            except OSError as e:
+                if e.errno == errno.ENOENT:
+                    raise ValueError('invalid process')
+                else:
+                    raise
         for p in ('rofs', 'rwfs', 'squashmnt', 'persistmnt'):
             if self['ExecutablePath'].startswith('/%s/' % p):
                 self['ExecutablePath'] = self['ExecutablePath'][len('/%s' % p):]
@@ -552,18 +602,17 @@ class Report(problem_report.ProblemReport):
             # On Linux 2.6.28+, 'current' is world readable, but read() gives
             # EPERM; Python 2.5.3+ crashes on that (LP: #314065)
             if os.getuid() == 0:
-                with open('/proc/' + pid + '/attr/current') as fd:
-                    val = fd.read().strip()
+                val = _read_proc_file('attr/current', pid, proc_pid_fd)
                 if val != 'unconfined':
                     self['ProcAttrCurrent'] = val
         except (IOError, OSError):
             pass
 
-        ret = self.get_logind_session(pid)
+        ret = self.get_logind_session(pid, proc_pid_fd)
         if ret:
             self['_LogindSession'] = ret[0]
 
-    def add_proc_environ(self, pid=None, extraenv=[]):
+    def add_proc_environ(self, pid=None, extraenv=[], proc_pid_fd=None):
         '''Add environment information.
 
         If pid is not given, it defaults to the process' current pid.
@@ -580,22 +629,25 @@ class Report(problem_report.ProblemReport):
                      'LC_TELEPHONE', 'LC_MEASUREMENT', 'LC_IDENTIFICATION',
                      'LOCPATH'] + extraenv
 
-        if not pid:
-            pid = os.getpid()
-        pid = str(pid)
+        if not proc_pid_fd:
+            if not pid:
+                pid = os.getpid()
+            pid = str(pid)
+            if not _python2:
+                proc_pid_fd = os.open('/proc/%s' % pid, os.O_RDONLY | os.O_PATH | os.O_DIRECTORY)
 
         self['ProcEnviron'] = ''
-        env = _read_file('/proc/' + pid + '/environ').replace('\n', '\\n')
+        env = _read_proc_file('environ', pid, proc_pid_fd).replace('\n', '\\n')
         if env.startswith('Error:'):
             self['ProcEnviron'] = env
         else:
-            for l in env.split('\0'):
-                if l.split('=', 1)[0] in safe_vars:
+            for line in env.split('\0'):
+                if line.split('=', 1)[0] in safe_vars:
                     if self['ProcEnviron']:
                         self['ProcEnviron'] += '\n'
-                    self['ProcEnviron'] += l
-                elif l.startswith('PATH='):
-                    p = l.split('=', 1)[1]
+                    self['ProcEnviron'] += line
+                elif line.startswith('PATH='):
+                    p = line.split('=', 1)[1]
                     if '/home' in p or '/tmp' in p:
                         if self['ProcEnviron']:
                             self['ProcEnviron'] += '\n'
@@ -604,20 +656,20 @@ class Report(problem_report.ProblemReport):
                         if self['ProcEnviron']:
                             self['ProcEnviron'] += '\n'
                         self['ProcEnviron'] += 'PATH=(custom, no user)'
-                elif l.startswith('XDG_RUNTIME_DIR='):
+                elif line.startswith('XDG_RUNTIME_DIR='):
                     if self['ProcEnviron']:
                         self['ProcEnviron'] += '\n'
                     self['ProcEnviron'] += 'XDG_RUNTIME_DIR=<set>'
-                elif l.startswith('LD_PRELOAD='):
+                elif line.startswith('LD_PRELOAD='):
                     if self['ProcEnviron']:
                         self['ProcEnviron'] += '\n'
                     self['ProcEnviron'] += 'LD_PRELOAD=<set>'
-                elif l.startswith('LD_LIBRARY_PATH='):
+                elif line.startswith('LD_LIBRARY_PATH='):
                     if self['ProcEnviron']:
                         self['ProcEnviron'] += '\n'
                     self['ProcEnviron'] += 'LD_LIBRARY_PATH=<set>'
-                elif l.startswith('XDG_CURRENT_DESKTOP='):
-                    self['CurrentDesktop'] = l.split('=', 1)[1]
+                elif line.startswith('XDG_CURRENT_DESKTOP='):
+                    self['CurrentDesktop'] = line.split('=', 1)[1]
 
     def add_kernel_crash_info(self, debugdir=None):
         '''Add information from kernel crash.
@@ -660,7 +712,7 @@ class Report(problem_report.ProblemReport):
                 os.unlink(core)
         return ret
 
-    def add_gdb_info(self, rootdir=None):
+    def add_gdb_info(self, rootdir=None, gdb_sandbox=None):
         '''Add information from gdb.
 
         This requires that the report has a CoreDump and an
@@ -679,7 +731,8 @@ class Report(problem_report.ProblemReport):
         chroot() or root privileges, it just instructs gdb to search for the
         files there.
 
-        Raises a IOError if the core dump is invalid/truncated.
+        Raises a IOError if the core dump is invalid/truncated, or OSError if
+        calling gdb fails.
         '''
         if 'CoreDump' not in self or 'ExecutablePath' not in self:
             return
@@ -691,8 +744,7 @@ class Report(problem_report.ProblemReport):
                        'AssertionMessage': 'print __abort_msg->msg',
                        'GLibAssertionMessage': 'print __glib_assert_msg',
                        'NihAssertionMessage': 'print (char*) __nih_abort_msg'}
-
-        gdb_cmd = self.gdb_command(rootdir)
+        gdb_cmd, environ = self.gdb_command(rootdir, gdb_sandbox)
 
         # limit maximum backtrace depth (to avoid looped stacks)
         gdb_cmd += ['--batch', '--ex', 'set backtrace limit 2000']
@@ -703,21 +755,20 @@ class Report(problem_report.ProblemReport):
             value_keys.append(name)
             gdb_cmd += ['--ex', 'p -99', '--ex', cmd]
 
-        # call gdb
-        try:
-            out = _command_output(gdb_cmd).decode('UTF-8', errors='replace')
-        except OSError:
-            return
+        out = _command_output(gdb_cmd, env=environ).decode('UTF-8', errors='replace')
 
         # check for truncated stack trace
         if 'is truncated: expected core file size' in out:
-            warnings = '\n'.join([l for l in out.splitlines() if 'Warning:' in l])
+            if 'warning:' in out:
+                warnings = '\n'.join([l for l in out.splitlines() if 'warning:' in l])
+            elif 'Warning:' in out:
+                warnings = '\n'.join([l for l in out.splitlines() if 'Warning:' in l])
             reason = 'Invalid core dump: ' + warnings.strip()
             self['UnreportableReason'] = reason
             raise IOError(reason)
 
         # split the output into the various fields
-        part_re = re.compile('^\$\d+\s*=\s*-99$', re.MULTILINE)
+        part_re = re.compile(r'^\$\d+\s*=\s*-99$', re.MULTILINE)
         parts = part_re.split(out)
         # drop the gdb startup text prior to first separator
         parts.pop(0)
@@ -740,7 +791,7 @@ class Report(problem_report.ProblemReport):
         # clean up AssertionMessage
         if 'AssertionMessage' in self:
             # chop off "$n = 0x...." prefix, drop empty ones
-            m = re.match('^\$\d+\s+=\s+0x[0-9a-fA-F]+\s+"(.*)"\s*$',
+            m = re.match(r'^\$\d+\s+=\s+0x[0-9a-fA-F]+\s+"(.*)"\s*$',
                          self['AssertionMessage'])
             if m:
                 self['AssertionMessage'] = m.group(1)
@@ -769,10 +820,10 @@ class Report(problem_report.ProblemReport):
         unwound = False
         unwinding = False
         unwinding_xerror = False
-        bt_fn_re = re.compile('^#(\d+)\s+(?:0x(?:\w+)\s+in\s+\*?(.*)|(<signal handler called>)\s*)$')
-        bt_fn_noaddr_re = re.compile('^#(\d+)\s+(?:(.*)|(<signal handler called>)\s*)$')
+        bt_fn_re = re.compile(r'^#(\d+)\s+(?:0x(?:\w+)\s+in\s+\*?(.*)|(<signal handler called>)\s*)$')
+        bt_fn_noaddr_re = re.compile(r'^#(\d+)\s+(?:(.*)|(<signal handler called>)\s*)$')
         # some internal functions like the SSE stubs cause unnecessary jitter
-        ignore_functions_re = re.compile('^(__.*_s?sse\d+(?:_\w+)?|__kernel_vsyscall)$')
+        ignore_functions_re = re.compile(r'^(__.*_s?sse\d+(?:_\w+)?|__kernel_vsyscall)$')
 
         for line in self['Stacktrace'].splitlines():
             m = bt_fn_re.match(line)
@@ -903,7 +954,7 @@ class Report(problem_report.ProblemReport):
                 <re key="Foo">ba.*r</re>
             </pattern>
             <pattern url="http://bugtracker.net/bugs/2">
-                <re key="Package">^\S* 1-2$</re> <!-- test for a particular version -->
+                <re key="Package">^\\S* 1-2$</re> <!-- test for a particular version -->
                 <re key="Foo">write_(hello|goodbye)</re>
             </pattern>
         </patterns>
@@ -942,15 +993,25 @@ class Report(problem_report.ProblemReport):
         ifpath = os.path.expanduser(_ignore_file)
         if orig_home is not None:
             os.environ['HOME'] = orig_home
-        if not os.access(ifpath, os.R_OK) or os.path.getsize(ifpath) == 0:
+        euid = os.geteuid()
+        try:
+            # drop permissions temporarily to try open users ignore file
+            os.seteuid(os.getuid())
+            fp = open(ifpath, 'r')
+        except (IOError, OSError):
+            fp = None
+        finally:
+            os.seteuid(euid)
+        if fp is None or os.fstat(fp.fileno()).st_size == 0:
             # create a document from scratch
             dom = xml.dom.getDOMImplementation().createDocument(None, 'apport', None)
         else:
             try:
-                dom = xml.dom.minidom.parse(ifpath)
+                dom = xml.dom.minidom.parse(fp)
             except ExpatError as e:
                 raise ValueError('%s has invalid format: %s' % (_ignore_file, str(e)))
-
+        if fp is not None:
+            fp.close()
         # remove whitespace so that writing back the XML does not accumulate
         # whitespace
         dom.documentElement.normalize()
@@ -1091,8 +1152,8 @@ class Report(problem_report.ProblemReport):
     def stacktrace_top_function(self):
         '''Return topmost function in StacktraceTop'''
 
-        for l in self.get('StacktraceTop', '').splitlines():
-            fname = l.split('(')[0].strip()
+        for line in self.get('StacktraceTop', '').splitlines():
+            fname = line.split('(')[0].strip()
             if fname != '??':
                 return fname
 
@@ -1154,7 +1215,7 @@ class Report(problem_report.ProblemReport):
                     os.path.basename(self['ExecutablePath']),
                     trace[0])
 
-            trace_re = re.compile('^\s*File\s*"(\S+)".* in (.+)$')
+            trace_re = re.compile(r'^\s*File\s*"(\S+)".* in (.+)$')
             i = len(trace) - 1
             function = 'unknown'
             while i >= 0:
@@ -1233,10 +1294,10 @@ class Report(problem_report.ProblemReport):
         '''Return list of obsolete packages in Package and Dependencies.'''
 
         obsolete = []
-        for l in (self.get('Package', '') + '\n' + self.get('Dependencies', '')).splitlines():
-            if not l:
+        for line in (self.get('Package', '') + '\n' + self.get('Dependencies', '')).splitlines():
+            if not line:
                 continue
-            pkg, ver = l.split()[:2]
+            pkg, ver = line.split()[:2]
             avail = packaging.get_available_version(pkg)
             if ver is not None and ver != 'None' and avail is not None and packaging.compare_versions(ver, avail) < 0:
                 obsolete.append(pkg)
@@ -1271,7 +1332,7 @@ class Report(problem_report.ProblemReport):
         # kernel crash
         if 'Stacktrace' in self and self['ProblemType'] == 'KernelCrash':
             sig = 'kernel'
-            regex = re.compile('^\s*\#\d+\s\[\w+\]\s(\w+)')
+            regex = re.compile(r'^\s*\#\d+\s\[\w+\]\s(\w+)')
             for line in self['Stacktrace'].splitlines():
                 m = regex.match(line)
                 if m:
@@ -1287,7 +1348,7 @@ class Report(problem_report.ProblemReport):
         # signal crashes
         if 'StacktraceTop' in self and 'Signal' in self:
             sig = '%s:%s' % (self['ExecutablePath'], self['Signal'])
-            bt_fn_re = re.compile('^(?:([\w:~]+).*|(<signal handler called>)\s*)$')
+            bt_fn_re = re.compile(r'^(?:([\w:~]+).*|(<signal handler called>)\s*)$')
 
             lines = self['StacktraceTop'].splitlines()
             if len(lines) < 2:
@@ -1309,7 +1370,7 @@ class Report(problem_report.ProblemReport):
             sig = ''
             if len(trace) == 1:
                 # sometimes, Python exceptions do not have file references
-                m = re.match('(\w+): ', trace[0])
+                m = re.match(r'(\w+): ', trace[0])
                 if m:
                     return self['ExecutablePath'] + ':' + m.group(1)
                 else:
@@ -1317,9 +1378,9 @@ class Report(problem_report.ProblemReport):
             elif len(trace) < 3:
                 return None
 
-            loc_re = re.compile('^\s+File "([^"]+).*line (\d+).*\sin (.*)$')
-            for l in trace:
-                m = loc_re.match(l)
+            loc_re = re.compile(r'^\s+File "([^"]+).*line (\d+).*\sin (.*)$')
+            for line in trace:
+                m = loc_re.match(line)
                 if m:
                     # if we have a function name, use this; for a a crash
                     # outside of a function/method, fall back to the source
@@ -1383,7 +1444,7 @@ class Report(problem_report.ProblemReport):
         return None
 
     def _extract_function_and_address(self, line):
-        parsed = re.search('\[.*\] (.*)$', line)
+        parsed = re.search(r'\[.*\] (.*)$', line)
         if parsed:
             match = parsed.group(1)
             assert match, 'could not parse expected call trace line: %s' % line
@@ -1492,7 +1553,7 @@ class Report(problem_report.ProblemReport):
                     else:
                         self[k] = pattern.sub(repl, self[k])
 
-    def gdb_command(self, sandbox):
+    def gdb_command(self, sandbox, gdb_sandbox=None):
         '''Build gdb command for this report.
 
         This builds a gdb command for processing the given report, by setting
@@ -1503,37 +1564,61 @@ class Report(problem_report.ProblemReport):
         When available, this calls "gdb-multiarch" instead of "gdb", for
         processing crash reports from foreign architectures.
 
-        Return argv list.
+        Return argv list for gdb and any environment variables.
         '''
         assert 'ExecutablePath' in self
         executable = self.get('InterpreterPath', self['ExecutablePath'])
 
-        command = ['gdb']
+        same_arch = False
+        if 'Architecture' in self and \
+                self['Architecture'] == packaging.get_system_architecture():
+            same_arch = True
 
-        if 'Architecture' in self and self['Architecture'] != packaging.get_system_architecture():
+        gdb_sandbox_bin = (os.path.join(gdb_sandbox, 'usr', 'bin')
+                           if gdb_sandbox else None)
+        gdb_path = _which_extrapath('gdb', gdb_sandbox_bin)
+        if not gdb_path:
+            apport.fatal('gdb does not exist in the %ssandbox nor on the host'
+                         % ('gdb ' if not same_arch else ''))
+        command = [gdb_path]
+        environ = None
+
+        if not same_arch:
             # check if we have gdb-multiarch
-            which = subprocess.Popen(['which', 'gdb-multiarch'],
-                                     stdout=subprocess.PIPE)
-            which.communicate()
-            if which.returncode == 0:
-                command = ['gdb-multiarch']
+            ma = _which_extrapath('gdb-multiarch', gdb_sandbox_bin)
+            if ma:
+                command = [ma]
             else:
                 sys.stderr.write(
                     'WARNING: Please install gdb-multiarch for processing '
                     'reports from foreign architectures. Results with "gdb" '
                     'will be very poor.\n')
 
-            # check for foreign architecture
-            arch = self.get('Uname', 'none').split()[-1]
-            if 'arm' in arch:
-                command += ['--ex', 'set architecture arm', '--ex', 'set gnutarget elf32-littlearm']
-            elif 'ppc' in arch:
-                command += ['--ex', 'set architecture powerpc:common', '--ex', 'set gnutarget elf32-powerpc']
-            # note, i386 vs. x86_64 is auto-detected just fine
-
         if sandbox:
-            command += ['--ex', 'set debug-file-directory %s/usr/lib/debug' % sandbox,
-                        '--ex', 'set solib-absolute-prefix ' + sandbox]
+            native_multiarch = "x86_64-linux-gnu"
+            # N.B. set solib-absolute-prefix is an alias for set sysroot
+            command += ['--ex',
+                        'set debug-file-directory %s/usr/lib/debug' % sandbox,
+                        '--ex', 'set solib-absolute-prefix ' + sandbox,
+                        '--ex', 'add-auto-load-safe-path ' + sandbox,
+                        # needed to fix /lib64/ld-linux-x86-64.so.2 broken
+                        # symlink
+                        '--ex', 'set solib-search-path %s/lib/%s' % \
+                        (sandbox, native_multiarch)]
+            if gdb_sandbox:
+                ld_lib_path = '%s/lib:%s/lib/%s:%s/usr/lib/%s:%s/usr/lib' % \
+                    (gdb_sandbox, gdb_sandbox, native_multiarch,
+                     gdb_sandbox, native_multiarch, gdb_sandbox)
+                pyhome = '%s/usr' % gdb_sandbox
+                # env settings need to be modified for gdb in a sandbox
+                environ = {'LD_LIBRARY_PATH': ld_lib_path,
+                           'PYTHONHOME': pyhome,
+                           'GCONV_PATH': '%s/usr/lib/%s/gconv' %
+                           (gdb_sandbox, native_multiarch)}
+                command.insert(0, '%s/lib/%s/ld-linux-x86-64.so.2' %
+                               (gdb_sandbox, native_multiarch))
+                command += ['--ex', 'set data-directory %s/usr/share/gdb' %
+                            gdb_sandbox]
             executable = sandbox + '/' + executable
 
         assert os.path.exists(executable)
@@ -1557,7 +1642,7 @@ class Report(problem_report.ProblemReport):
 
             command += ['--ex', 'core-file ' + core]
 
-        return command
+        return command, environ
 
     def _address_to_offset(self, addr):
         '''Resolve a memory address to an ELF name and offset.
@@ -1596,8 +1681,8 @@ class Report(problem_report.ProblemReport):
         # data field and the path there are many spaces, while between the
         # other data fields there is only one. So we take 2 or more spaces as
         # the separator of the last data field and the path.
-        fmt = re.compile('^([0-9a-fA-F]+)-([0-9a-fA-F]+).*\s{2,}(\S.*$)')
-        fmt_unknown = re.compile('^([0-9a-fA-F]+)-([0-9a-fA-F]+)\s')
+        fmt = re.compile(r'^([0-9a-fA-F]+)-([0-9a-fA-F]+).*\s{2,}(\S.*$)')
+        fmt_unknown = re.compile(r'^([0-9a-fA-F]+)-([0-9a-fA-F]+)\s')
 
         for line in self['ProcMaps'].splitlines():
             if not line.strip():
@@ -1614,19 +1699,24 @@ class Report(problem_report.ProblemReport):
                                           int(m.group(2), 16), m.group(3)))
 
     @classmethod
-    def get_logind_session(klass, pid):
+    def get_logind_session(klass, pid=None, proc_pid_fd=None):
         '''Get logind session path and start time.
 
-        Return (session_id, session_start_timestamp) if pid is in a logind
+        Return (session_id, session_start_timestamp) if process is in a logind
         session, or None otherwise.
         '''
+        if not _python2 and proc_pid_fd is not None:
+            cgroup_file = os.open('cgroup', os.O_RDONLY | os.O_CLOEXEC, dir_fd=proc_pid_fd)
+        else:
+            cgroup_file = "/proc/%s/cgroup" % pid
+
         # determine cgroup
         try:
-            with open('/proc/%s/cgroup' % pid) as f:
-                for l in f:
-                    l = l.strip()
-                    if 'name=systemd:' in l and l.endswith('.scope') and '/session-' in l:
-                        my_session = l.split('/session-', 1)[1][:-6]
+            with io.open(cgroup_file) as f:
+                for line in f:
+                    line = line.strip()
+                    if 'name=systemd:' in line and line.endswith('.scope') and '/session-' in line:
+                        my_session = line.split('/session-', 1)[1][:-6]
                         break
                 else:
                     return None
