@@ -33,7 +33,7 @@ else:
     from configparser import ConfigParser
     PY3 = True
 
-__version__ = '2.20.1'
+__version__ = '2.20.9'
 
 
 def excstr(exception):
@@ -43,6 +43,7 @@ def excstr(exception):
         return str(exception).decode(locale.getpreferredencoding(), 'replace')
     return str(exception)
 
+
 symptom_script_dir = os.environ.get('APPORT_SYMPTOMS_DIR',
                                     '/usr/share/apport/symptoms')
 PF_KTHREAD = 0x200000
@@ -65,6 +66,19 @@ def still_running(pid):
     return True
 
 
+def find_snap(package):
+    import requests_unixsocket
+
+    session = requests_unixsocket.Session()
+    try:
+        r = session.get('http+unix://%2Frun%2Fsnapd.socket/v2/snaps/{}'.format(package))
+        if r.status_code == 200:
+            j = r.json()
+            return j["result"]
+    except Exception:
+        return None
+
+
 def thread_collect_info(report, reportfile, package, ui, symptom_script=None,
                         ignore_uninstalled=False):
     '''Collect information about report.
@@ -78,7 +92,12 @@ def thread_collect_info(report, reportfile, package, ui, symptom_script=None,
 
     If symptom_script is given, it will be run first (for run_symptom()).
     '''
-    report.add_gdb_info()
+    try:
+        report.add_gdb_info()
+    except OSError:
+        # it's okay if gdb is not installed on the client side; we'll get stack
+        # traces on retracing.
+        pass
     report.add_os_info()
 
     if symptom_script:
@@ -93,7 +112,7 @@ def thread_collect_info(report, reportfile, package, ui, symptom_script=None,
             report['Symptom'] = os.path.splitext(os.path.basename(symptom_script))[0]
         except StopIteration:
             sys.exit(0)
-        except:
+        except Exception as e:
             apport.error('symptom script %s crashed:', symptom_script)
             traceback.print_exc()
             sys.exit(0)
@@ -184,6 +203,7 @@ class UserInterface:
         self.report_file = None
         self.cur_package = None
         self.offer_restart = False
+        self.specified_a_pkg = False
 
         try:
             self.crashdb = apport.crashdb.get_crashdb(None)
@@ -238,7 +258,7 @@ class UserInterface:
             # reported in the same session if the logind session paths agree
             # and the session started before the report's "Date".
             if logind_session and '_LogindSession' in self.report and \
-                    'Date' in self.report:
+               'Date' in self.report:
                 if logind_session[0] != self.report['_LogindSession'] or \
                    logind_session[1] > self.report.get_timestamp():
                     continue
@@ -303,6 +323,13 @@ class UserInterface:
                 self.restart()
             if response['blacklist']:
                 self.report.mark_ignore()
+            try:
+                if response['remember']:
+                    self.remember_send_report(response['report'])
+            # use try/expect for python2 support. Old reports (generated pre-apport 2.20.10-0ubuntu4)
+            # may not have the remember key and can be loaded afterwards (or after dist-upgrade)
+            except KeyError:
+                pass
             if not response['report']:
                 return
 
@@ -501,7 +528,7 @@ class UserInterface:
                 self.ui_error_message(_('Cannot create report'), excstr(e))
         else:
             # show what's being sent
-            allowed_to_report = apport.fileutils.allowed_to_report()
+            allowed_to_report = True
             response = self.ui_present_report_details(allowed_to_report)
             if response['report']:
                 self.file_report()
@@ -568,7 +595,6 @@ class UserInterface:
         self.add_extra_tags()
 
         # delete the uninteresting keys
-        del self.report['ProblemType']
         del self.report['Date']
         try:
             del self.report['SourcePackage']
@@ -581,7 +607,7 @@ class UserInterface:
             return False
 
         # show what's being sent
-        allowed_to_report = apport.fileutils.allowed_to_report()
+        allowed_to_report = True
         response = self.ui_present_report_details(allowed_to_report)
         if response['report']:
             self.crashdb.update(self.options.update_report, self.report,
@@ -608,7 +634,7 @@ class UserInterface:
             try:
                 with open(script) as f:
                     exec(compile(f.read(), script, 'exec'), symb)
-            except:
+            except Exception:
                 apport.error('symptom script %s is invalid', script)
                 traceback.print_exc()
                 continue
@@ -812,10 +838,21 @@ class UserInterface:
 
         # executable?
         elif '/' in self.args[0]:
-            pkg = apport.packaging.get_file_package(self.args[0])
-            if not pkg:
-                optparser.error('%s does not belong to a package.' % self.args[0])
+            if self.args[0].startswith('/snap/bin'):
+                # see if the snap has the same name as the executable
+                snap = find_snap(self.args[0].split('/')[-1])
+                if not snap:
+                    optparser.error('%s is provided by a snap. No contact address has been provided; visit the forum at https://forum.snapcraft.io/ for help.' % self.args[0])
+                elif snap.get("contact", ""):
+                    optparser.error('%s is provided by a snap published by %s. Contact them via %s for help.' % (self.args[0], snap["developer"], snap["contact"]))
+                else:
+                    optparser.error('%s is provided by a snap published by %s. No contact address has been provided; visit the forum at https://forum.snapcraft.io/ for help.' % (self.args[0], snap["developer"]))
                 sys.exit(1)
+            else:
+                pkg = apport.packaging.get_file_package(self.args[0])
+                if not pkg:
+                    optparser.error('%s does not belong to a package.' % self.args[0])
+                    sys.exit(1)
             self.args = []
             self.options.filebug = True
             self.options.package = pkg
@@ -823,6 +860,7 @@ class UserInterface:
         # otherwise: package name
         else:
             self.options.filebug = True
+            self.specified_a_pkg = True
             self.options.package = self.args[0]
             self.args = []
 
@@ -926,6 +964,26 @@ class UserInterface:
 
         self.ui_run_terminal(cmds[response[0]])
 
+    def remember_send_report(self, send_report):
+        '''Put whoopsie in auto or never mode'''
+        if send_report:
+            send_report = "true"
+        else:
+            send_report = "false"
+        try:
+            subprocess.check_output(['/usr/bin/gdbus', 'call', '-y',
+                                     '-d', 'com.ubuntu.WhoopsiePreferences',
+                                     '-o', '/com/ubuntu/WhoopsiePreferences',
+                                     '-m', 'com.ubuntu.WhoopsiePreferences.SetReportCrashes', send_report])
+            subprocess.check_output(['/usr/bin/gdbus', 'call', '-y',
+                                     '-d', 'com.ubuntu.WhoopsiePreferences',
+                                     '-o', '/com/ubuntu/WhoopsiePreferences',
+                                     '-m', 'com.ubuntu.WhoopsiePreferences.SetAutomaticallyReportCrashes', 'true'])
+        except (OSError, subprocess.CalledProcessError) as e:
+            self.ui_error_message(_("Can't remember send report status settings"), '%s\n\n%s' % (
+                                  _("Saving crash reporting state failed. Can't set auto or never reporting mode."),
+                                  excstr(e)))
+
     def check_report_crashdb(self):
         '''Process reports' CrashDB field, if present'''
 
@@ -1058,14 +1116,37 @@ class UserInterface:
                         repr(e))
                     self.report['_MarkForUpload'] = 'False'
                 except ValueError:  # package does not exist
-                    self.report['UnreportableReason'] = _('The report belongs to a package that is not installed.')
-                    self.report['_MarkForUpload'] = 'False'
+                    snap = find_snap(self.cur_package)
+                    if not snap:
+                        pass
+                    elif snap.get("contact", ""):
+                        self.report['UnreportableReason'] = _('This report is about a snap published by %s. Contact them via %s for help.') % (snap["developer"], snap["contact"])
+                        self.report['_MarkForUpload'] = 'False'
+                    else:
+                        self.report['UnreportableReason'] = _('This report is about a snap published by %s. No contact address has been provided; visit the forum at https://forum.snapcraft.io/ for help.') % snap["developer"]
+                        self.report['_MarkForUpload'] = 'False'
+
+                    if 'UnreportableReason' not in self.report:
+                        self.report['UnreportableReason'] = _('This report is about a package that is not installed.')
+                        self.report['_MarkForUpload'] = 'False'
                 except Exception as e:
                     apport.error(repr(e))
                     self.report['UnreportableReason'] = _('An error occurred while attempting to '
                                                           'process this problem report:') + '\n\n' + str(e)
                     self.report['_MarkForUpload'] = 'False'
 
+            snap = find_snap(self.cur_package)
+            if snap and 'UnreportableReason' not in self.report and self.specified_a_pkg:
+                if snap.get("contact", ""):
+                    msg = _('You are about to report a bug against the deb package, but you also a have snap published by %s installed. You can contact them via %s for help. Do you want to continue with the bug report against the deb?') % (snap["developer"], snap["contact"])
+                else:
+                    msg = _('You are about to report a bug against the deb package, but you also a have snap published by %s installed. For the snap, no contact address has been provided; visit the forum at https://forum.snapcraft.io/ for help. Do you want to continue with the bug report against the deb?') % snap["developer"]
+
+                if not self.ui_question_yesno(msg):
+                    self.ui_stop_info_collection_progress()
+                    sys.exit(0)
+                    return
+
             if 'UnreportableReason' in self.report or not self.check_report_crashdb():
                 self.ui_stop_info_collection_progress()
                 if on_finished:
