@@ -26,9 +26,30 @@ import os
 import warnings
 import weakref
 
+try:
+    from typing import (Any, Callable, Dict, Iterator, List, Optional,
+                        Set, Tuple, cast)
+    Any  # pyflakes
+    Callable  # pyflakes
+    Dict  # pyflakes
+    Iterator  # pyflakes
+    List  # pyflakes
+    Optional  # pyflakes
+    Set  # pyflakes
+    Tuple  # pyflakes
+except ImportError:
+    def cast(typ, obj):  # type: ignore
+        return obj
+    pass
+
 import apt_pkg
-from apt import Package
+from apt.package import Package, Version
 import apt.progress.text
+from apt.progress.base import AcquireProgress, InstallProgress, OpProgress
+OpProgress  # pyflakes
+InstallProgress  # pyflakes
+AcquireProgress  # pyflakes
+Version  # pyflakes
 
 
 class FetchCancelledException(IOError):
@@ -47,29 +68,6 @@ class CacheClosedException(Exception):
     """Exception that is thrown when the cache is used after close()."""
 
 
-class _WrappedLock(object):
-    """Wraps an apt_pkg.FileLock to raise LockFailedException.
-
-    Initialized using a directory path."""
-
-    def __init__(self, path):
-        # type: (str) -> None
-        self._path = path
-        self._lock = apt_pkg.FileLock(os.path.join(path, "lock"))
-
-    def __enter__(self):
-        # type: () -> None
-        try:
-            return self._lock.__enter__()
-        except apt_pkg.Error as e:
-            raise LockFailedException(("Failed to lock directory %s: %s") %
-                                       (self._path, e))
-
-    def __exit__(self, typ, value, traceback):
-        # type: (object, object, object) -> None
-        return self._lock.__exit__(typ, value, traceback)
-
-
 class Cache(object):
     """Dictionary-like package cache.
 
@@ -97,16 +95,17 @@ class Cache(object):
     """
 
     def __init__(self, progress=None, rootdir=None, memonly=False):
-        self._cache = None
-        self._depcache = None
-        self._records = None
-        self._list = None
-        self._callbacks = {}
-        self._callbacks2 = {}
-        self._weakref = weakref.WeakValueDictionary()
+        # type: (OpProgress, str, bool) -> None
+        self._cache = cast(apt_pkg.Cache, None)  # type: apt_pkg.Cache
+        self._depcache = cast(apt_pkg.DepCache, None)  # type: apt_pkg.DepCache
+        self._records = cast(apt_pkg.PackageRecords, None)  # type: apt_pkg.PackageRecords # nopep8
+        self._list = cast(apt_pkg.SourceList, None)  # type: apt_pkg.SourceList
+        self._callbacks = {}  # type: Dict[str, List[Callable[..., None]]]
+        self._callbacks2 = {}  # type: Dict[str, List[Tuple[Callable[..., None], List[Any], Dict[Any,Any]]]] # nopep8
+        self._weakref = weakref.WeakValueDictionary()  # type: weakref.WeakValueDictionary[str, apt.Package] # nopep8
         self._weakversions = weakref.WeakSet()  # type: weakref.WeakSet[Version] # nopep8
         self._changes_count = -1
-        self._sorted_set = None
+        self._sorted_set = None  # type: Optional[List[str]]
 
         self.connect("cache_post_open", "_inc_changes_count")
         self.connect("cache_post_change", "_inc_changes_count")
@@ -134,18 +133,15 @@ class Cache(object):
             # Call InitSystem so the change to Dir::State::Status is actually
             # recognized (LP: #320665)
             apt_pkg.init_system()
-
-        # Prepare a lock object (context manager for archive lock)
-        archive_dir = apt_pkg.config.find_dir("Dir::Cache::Archives")
-        self._archive_lock = _WrappedLock(archive_dir)
-
         self.open(progress)
 
     def _inc_changes_count(self):
+        # type: () -> None
         """Increase the number of changes"""
         self._changes_count += 1
 
     def _check_and_create_required_dirs(self, rootdir):
+        # type: (str) -> None
         """
         check if the required apt directories/files are there and if
         not create them
@@ -167,6 +163,7 @@ class Cache(object):
                 open(rootdir + f, "w").close()
 
     def _run_callbacks(self, name):
+        # type: (str) -> None
         """ internal helper to run a callback """
         if name in self._callbacks:
             for callback in self._callbacks[name]:
@@ -180,6 +177,7 @@ class Cache(object):
                     callback(self, *args, **kwds)
 
     def open(self, progress=None):
+        # type: (OpProgress) -> None
         """ Open the package cache, after that it can be used like
             a dictionary
         """
@@ -236,12 +234,14 @@ class Cache(object):
                 self._weakversions.remove(ver)
 
     def close(self):
+        # type: () -> None
         """ Close the package cache """
         # explicitely free the FDs that _records has open
         del self._records
-        self._records = None
+        self._records = cast(apt_pkg.PackageRecords, None)
 
     def __enter__(self):
+        # type: () -> Cache
         """ Enter the with statement """
         return self
 
@@ -250,11 +250,12 @@ class Cache(object):
         self.close()
 
     def __getitem__(self, key):
+        # type: (object) -> Package
         """ look like a dictionary (get key) """
+        key = str(key)
         try:
             return self._weakref[key]
         except KeyError:
-            key = str(key)
             try:
                 rawpkg = self._cache[key]
             except KeyError:
@@ -269,6 +270,17 @@ class Cache(object):
 
             return pkg
 
+    def get(self, key, default=None):
+        # type: (object, object) -> Any
+        """Return *self*[*key*] or *default* if *key* not in *self*.
+
+        .. versionadded:: 1.1
+        """
+        try:
+            return self[key]
+        except KeyError:
+            return default
+
     def _rawpkg_to_pkg(self, rawpkg):
         """Returns the apt.Package object for an apt_pkg.Package object.
 
@@ -283,6 +295,7 @@ class Cache(object):
         return pkg
 
     def __iter__(self):
+        # type: () -> Iterator[Package]
         # We iterate sorted over package names here. With this we read the
         # package lists linearly if we need to access the package records,
         # instead of having to do thousands of random seeks; the latter
@@ -296,18 +309,22 @@ class Cache(object):
         return rawpkg.has_versions
 
     def has_key(self, key):
+        # type: (object) -> bool
         return key in self
 
     def __contains__(self, key):
+        # type: (object) -> bool
         try:
             return self.__is_real_pkg(self._cache[key])
         except KeyError:
             return False
 
     def __len__(self):
+        # type: () -> int
         return len(self.keys())
 
     def keys(self):
+        # FIXME: type: () -> List[str] - does not work
         if self._sorted_set is None:
             self._sorted_set = sorted(p.get_fullname(pretty=True)
                                       for p in self._cache.packages
@@ -315,6 +332,7 @@ class Cache(object):
         return list(self._sorted_set)  # We need a copy here, caller may modify
 
     def get_changes(self):
+        # type: () -> List[Package]
         """ Get the marked changes """
         changes = []
         marked_keep = self._depcache.marked_keep
@@ -324,6 +342,7 @@ class Cache(object):
         return changes
 
     def upgrade(self, dist_upgrade=False):
+        # type: (bool) -> None
         """Upgrade all packages.
 
         If the parameter *dist_upgrade* is True, new dependencies will be
@@ -336,6 +355,7 @@ class Cache(object):
 
     @property
     def required_download(self):
+        # type: () -> int
         """Get the size of the packages that are required to download."""
         if self._records is None:
             raise CacheClosedException(
@@ -347,11 +367,13 @@ class Cache(object):
 
     @property
     def required_space(self):
+        # type: () -> int
         """Get the size of the additional required space on the fs."""
         return self._depcache.usr_size
 
     @property
     def req_reinstall_pkgs(self):
+        # type: () -> Set[str]
         """Return the packages not downloadable packages in reqreinst state."""
         reqreinst = set()
         get_candidate_ver = self._depcache.get_candidate_ver
@@ -364,6 +386,7 @@ class Cache(object):
         return reqreinst
 
     def _run_fetcher(self, fetcher):
+        # type: (apt_pkg.Acquire) -> int
         # do the actual fetching
         res = fetcher.run()
 
@@ -387,6 +410,7 @@ class Cache(object):
         return res
 
     def _fetch_archives(self, fetcher, pm):
+        # type: (apt_pkg.Acquire, apt_pkg.PackageManager) -> int
         """ fetch the needed archives """
         if self._records is None:
             raise CacheClosedException(
@@ -399,14 +423,25 @@ class Cache(object):
         # fetched
         return self._run_fetcher(fetcher)
 
+    def _get_archive_lock(self, fetcher):
+        # type: (apt_pkg.Acquire) -> None
+        # get lock
+        archive_dir = apt_pkg.config.find_dir("Dir::Cache::Archives")
+        try:
+            fetcher.get_lock(archive_dir)
+        except apt_pkg.Error as e:
+            raise LockFailedException(("Failed to lock archive directory %s: "
+                                       " %s") % (archive_dir, e))
+
     def fetch_archives(self, progress=None, fetcher=None):
+        # type: (AcquireProgress, apt_pkg.Acquire) -> int
         """Fetch the archives for all packages marked for install/upgrade.
 
         You can specify either an :class:`apt.progress.base.AcquireProgress()`
         object for the parameter *progress*, or specify an already
         existing :class:`apt_pkg.Acquire` object for the parameter *fetcher*.
 
-        The return value of the function is undefined. If an error occured,
+        The return value of the function is undefined. If an error occurred,
         an exception of type :class:`FetchFailedException` or
         :class:`FetchCancelledException` is raised.
 
@@ -419,11 +454,13 @@ class Cache(object):
         if fetcher is None:
             fetcher = apt_pkg.Acquire(progress)
 
-        with self._archive_lock:
-            return self._fetch_archives(fetcher,
-                                        apt_pkg.PackageManager(self._depcache))
+        self._get_archive_lock(fetcher)
+
+        return self._fetch_archives(fetcher,
+                                    apt_pkg.PackageManager(self._depcache))
 
     def is_virtual_package(self, pkgname):
+        # type: (str) -> bool
         """Return whether the package is a virtual package."""
         try:
             pkg = self._cache[pkgname]
@@ -434,6 +471,7 @@ class Cache(object):
 
     def get_providing_packages(self, pkgname, candidate_only=True,
                                include_nonvirtual=False):
+        # type: (str, bool, bool) -> List[Package]
         """Return a list of all packages providing a package.
 
         Return a list of packages which provide the virtual package of the
@@ -449,7 +487,7 @@ class Cache(object):
         a virtual pkg.
         """
 
-        providers = set()
+        providers = set()  # type: Set[Package]
         get_candidate_ver = self._depcache.get_candidate_ver
         try:
             vp = self._cache[pkgname]
@@ -466,6 +504,7 @@ class Cache(object):
 
     def update(self, fetch_progress=None, pulse_interval=0,
                raise_on_error=True, sources_list=None):
+        # FIXME: type: (AcquireProgress, int, bool, str) -> int
         """Run the equivalent of apt-get update.
 
         You probably want to call open() afterwards, in order to utilise the
@@ -478,43 +517,46 @@ class Cache(object):
         sources_list -- Update a alternative sources.list than the default.
         Note that the sources.list.d directory is ignored in this case
         """
-        with _WrappedLock(apt_pkg.config.find_dir("Dir::State::Lists")):
-            if sources_list:
-                old_sources_list = apt_pkg.config.find("Dir::Etc::sourcelist")
-                old_sources_list_d = (
-                    apt_pkg.config.find("Dir::Etc::sourceparts"))
-                old_cleanup = apt_pkg.config.find("APT::List-Cleanup")
-                apt_pkg.config.set("Dir::Etc::sourcelist",
-                                   os.path.abspath(sources_list))
-                apt_pkg.config.set("Dir::Etc::sourceparts", "xxx")
-                apt_pkg.config.set("APT::List-Cleanup", "0")
-                slist = apt_pkg.SourceList()
-                slist.read_main_list()
-            else:
-                slist = self._list
+        lockfile = apt_pkg.config.find_dir("Dir::State::Lists") + "lock"
+        lock = apt_pkg.get_lock(lockfile)
 
+        if lock < 0:
+            raise LockFailedException("Failed to lock %s" % lockfile)
+
+        if sources_list:
+            old_sources_list = apt_pkg.config.find("Dir::Etc::sourcelist")
+            old_sources_list_d = apt_pkg.config.find("Dir::Etc::sourceparts")
+            old_cleanup = apt_pkg.config.find("APT::List-Cleanup")
+            apt_pkg.config.set("Dir::Etc::sourcelist",
+                               os.path.abspath(sources_list))
+            apt_pkg.config.set("Dir::Etc::sourceparts", "xxx")
+            apt_pkg.config.set("APT::List-Cleanup", "0")
+            slist = apt_pkg.SourceList()
+            slist.read_main_list()
+        else:
+            slist = self._list
+
+        try:
+            if fetch_progress is None:
+                fetch_progress = apt.progress.base.AcquireProgress()
             try:
-                if fetch_progress is None:
-                    fetch_progress = apt.progress.base.AcquireProgress()
-                try:
-                    res = self._cache.update(fetch_progress, slist,
-                                             pulse_interval)
-                except SystemError as e:
-                    raise FetchFailedException(e)
-                if not res and raise_on_error:
-                    raise FetchFailedException()
-                else:
-                    return res
-            finally:
-                if sources_list:
-                    apt_pkg.config.set("Dir::Etc::sourcelist",
-                                       old_sources_list)
-                    apt_pkg.config.set("Dir::Etc::sourceparts",
-                                       old_sources_list_d)
-                    apt_pkg.config.set("APT::List-Cleanup",
-                                       old_cleanup)
+                res = self._cache.update(fetch_progress, slist,
+                                         pulse_interval)
+            except SystemError as e:
+                raise FetchFailedException(e)
+            if not res and raise_on_error:
+                raise FetchFailedException()
+            else:
+                return res
+        finally:
+            os.close(lock)
+            if sources_list:
+                apt_pkg.config.set("Dir::Etc::sourcelist", old_sources_list)
+                apt_pkg.config.set("Dir::Etc::sourceparts", old_sources_list_d)
+                apt_pkg.config.set("APT::List-Cleanup", old_cleanup)
 
     def install_archives(self, pm, install_progress):
+        # type: (apt_pkg.PackageManager, InstallProgress) -> int
         """
         The first parameter *pm* refers to an object returned by
         apt_pkg.PackageManager().
@@ -527,7 +569,7 @@ class Cache(object):
         """
         # compat with older API
         try:
-            install_progress.startUpdate()
+            install_progress.startUpdate()  # type: ignore
         except AttributeError:
             install_progress.start_update()
 
@@ -542,12 +584,13 @@ class Cache(object):
                 apt_pkg.pkgsystem_lock_inner()
 
         try:
-            install_progress.finishUpdate()
+            install_progress.finishUpdate()  # type: ignore
         except AttributeError:
             install_progress.finish_update()
         return res
 
     def commit(self, fetch_progress=None, install_progress=None):
+        # type: (AcquireProgress, InstallProgress) -> bool
         """Apply the marked changes to the cache.
 
         The first parameter, *fetch_progress*, refers to a FetchProgress()
@@ -574,38 +617,41 @@ class Cache(object):
         with apt_pkg.SystemLock():
             pm = apt_pkg.PackageManager(self._depcache)
             fetcher = apt_pkg.Acquire(fetch_progress)
-            with self._archive_lock:
-                while True:
-                    # fetch archives first
-                    res = self._fetch_archives(fetcher, pm)
+            self._get_archive_lock(fetcher)
 
-                    # then install
-                    res = self.install_archives(pm, install_progress)
-                    if res == pm.RESULT_COMPLETED:
-                        break
-                    elif res == pm.RESULT_FAILED:
-                        raise SystemError("installArchives() failed")
-                    elif res == pm.RESULT_INCOMPLETE:
-                        pass
-                    else:
-                        raise SystemError("internal-error: unknown result "
-                                          "code from InstallArchives: %s" %
-                                          res)
-                    # reload the fetcher for media swaping
-                    fetcher.shutdown()
+            while True:
+                # fetch archives first
+                res = self._fetch_archives(fetcher, pm)
+
+                # then install
+                res = self.install_archives(pm, install_progress)
+                if res == pm.RESULT_COMPLETED:
+                    break
+                elif res == pm.RESULT_FAILED:
+                    raise SystemError("installArchives() failed")
+                elif res == pm.RESULT_INCOMPLETE:
+                    pass
+                else:
+                    raise SystemError("internal-error: unknown result code "
+                                      "from InstallArchives: %s" % res)
+                # reload the fetcher for media swaping
+                fetcher.shutdown()
         return (res == pm.RESULT_COMPLETED)
 
     def clear(self):
+        # type: () -> None
         """ Unmark all changes """
         self._depcache.init()
 
     # cache changes
 
     def cache_post_change(self):
+        # type: () -> None
         " called internally if the cache has changed, emit a signal then "
         self._run_callbacks("cache_post_change")
 
     def cache_pre_change(self):
+        # type: () -> None
         """ called internally if the cache is about to change, emit
             a signal then """
         self._run_callbacks("cache_pre_change")
@@ -646,6 +692,7 @@ class Cache(object):
         self._callbacks2[name].append((callback, args, kwds))
 
     def actiongroup(self):
+        # type: () -> apt_pkg.ActionGroup
         """Return an `ActionGroup` object for the current cache.
 
         Action groups can be used to speedup actions. The action group is
@@ -668,6 +715,7 @@ class Cache(object):
 
     @property
     def dpkg_journal_dirty(self):
+        # type: () -> bool
         """Return True if the dpkg was interrupted
 
         All dpkg operations will fail until this is fixed, the action to
@@ -683,21 +731,25 @@ class Cache(object):
 
     @property
     def broken_count(self):
+        # type: () -> int
         """Return the number of packages with broken dependencies."""
         return self._depcache.broken_count
 
     @property
     def delete_count(self):
+        # type: () -> int
         """Return the number of packages marked for deletion."""
         return self._depcache.del_count
 
     @property
     def install_count(self):
+        # type: () -> int
         """Return the number of packages marked for installation."""
         return self._depcache.inst_count
 
     @property
     def keep_count(self):
+        # type: () -> int
         """Return the number of packages marked as keep."""
         return self._depcache.keep_count
 
@@ -709,32 +761,39 @@ class ProblemResolver(object):
     """
 
     def __init__(self, cache):
+        # type: (Cache) -> None
         self._resolver = apt_pkg.ProblemResolver(cache._depcache)
         self._cache = cache
 
     def clear(self, package):
+        # type: (Package) -> None
         """Reset the package to the default state."""
         self._resolver.clear(package._pkg)
 
     def install_protect(self):
+        # type: () -> None
         """mark protected packages for install or removal."""
         self._resolver.install_protect()
 
     def protect(self, package):
+        # type: (Package) -> None
         """Protect a package so it won't be removed."""
         self._resolver.protect(package._pkg)
 
     def remove(self, package):
+        # type: (Package) -> None
         """Mark a package for removal."""
         self._resolver.remove(package._pkg)
 
     def resolve(self):
+        # type: () -> None
         """Resolve dependencies, try to remove packages where needed."""
         self._cache.cache_pre_change()
         self._resolver.resolve()
         self._cache.cache_post_change()
 
     def resolve_by_keep(self):
+        # type: () -> None
         """Resolve dependencies, do not try to remove packages."""
         self._cache.cache_pre_change()
         self._resolver.resolve_by_keep()
@@ -748,6 +807,7 @@ class Filter(object):
     """ Filter base class """
 
     def apply(self, pkg):
+        # type: (Package) -> bool
         """ Filter function, return True if the package matchs a
             filter criteria and False otherwise
         """
@@ -758,6 +818,7 @@ class MarkedChangesFilter(Filter):
     """ Filter that returns all marked changes """
 
     def apply(self, pkg):
+        # type: (Package) -> bool
         if pkg.marked_install or pkg.marked_delete or pkg.marked_upgrade:
             return True
         else:
@@ -771,6 +832,7 @@ class InstalledFilter(Filter):
     """
 
     def apply(self, pkg):
+        # type: (Package) -> bool
         return pkg.is_installed
 
 
@@ -778,29 +840,33 @@ class _FilteredCacheHelper(object):
     """Helper class for FilteredCache to break a reference cycle."""
 
     def __init__(self, cache):
+        # type: (Cache) -> None
         # Do not keep a reference to the cache, or you have a cycle!
 
-        self._filtered = {}
-        self._filters = {}
+        self._filtered = {}  # type: Dict[str,bool]
+        self._filters = []  # type: List[Filter]
         cache.connect2("cache_post_change", self.filter_cache_post_change)
         cache.connect2("cache_post_open", self.filter_cache_post_change)
 
     def _reapply_filter(self, cache):
+        # type: (Cache) -> None
         " internal helper to refilter "
         # Do not keep a reference to the cache, or you have a cycle!
         self._filtered = {}
         for pkg in cache:
             for f in self._filters:
                 if f.apply(pkg):
-                    self._filtered[pkg.name] = 1
+                    self._filtered[pkg.name] = True
                     break
 
     def set_filter(self, filter):
+        # type: (Filter) -> None
         """Set the current active filter."""
         self._filters = []
         self._filters.append(filter)
 
     def filter_cache_post_change(self, cache):
+        # type: (Cache) -> None
         """Called internally if the cache changes, emit a signal then."""
         # Do not keep a reference to the cache, or you have a cycle!
         self._reapply_filter(cache)
@@ -813,6 +879,7 @@ class FilteredCache(object):
     """
 
     def __init__(self, cache=None, progress=None):
+        # type: (Cache, OpProgress) -> None
         if cache is None:
             self.cache = Cache(progress)
         else:
@@ -820,22 +887,28 @@ class FilteredCache(object):
         self._helper = _FilteredCacheHelper(self.cache)
 
     def __len__(self):
+        # type: () -> int
         return len(self._helper._filtered)
 
     def __getitem__(self, key):
+        # type: (str) -> Package
         return self.cache[key]
 
     def __iter__(self):
+        # type: () -> Iterator[Package]
         for pkgname in self._helper._filtered:
             yield self.cache[pkgname]
 
     def keys(self):
+        # FIXME: type: () -> List[str] - does not work
         return self._helper._filtered.keys()
 
     def has_key(self, key):
+        # type: (object) -> bool
         return key in self
 
     def __contains__(self, key):
+        # type: (object) -> bool
         try:
             # Normalize package name for multi arch
             return self.cache[key].name in self._helper._filtered
@@ -843,11 +916,13 @@ class FilteredCache(object):
             return False
 
     def set_filter(self, filter):
+        # type: (Filter) -> None
         """Set the current active filter."""
         self._helper.set_filter(filter)
         self.cache.cache_post_change()
 
     def filter_cache_post_change(self):
+        # type: () -> None
         """Called internally if the cache changes, emit a signal then."""
         self._helper.filter_cache_post_change(self.cache)
 
@@ -918,5 +993,7 @@ def _test():
         assert pkgname == filtered[pkgname].name
 
     print(len(filtered))
+
+
 if __name__ == '__main__':
     _test()
