@@ -500,7 +500,9 @@ class DebPackage(object):
                 self._dbg(1, "Found multiarch arch: '%s'" % arch)
             else:
                 self._dbg(1, "ERROR: Wrong architecture dude!")
-                self._failure_string = _("Wrong architecture '%s'") % arch
+                self._failure_string = _("Wrong architecture '%s' "
+                                         "-- Run dpkg --add-architecture to "
+                                         "add it and update afterwards") % arch
                 return False
 
         # check version
@@ -802,5 +804,6 @@ def _test():
     d = "libc6 (>= 2.3.2), libaio (>= 0.3.96) | libaio1 (>= 0.3.96)"
     print(s._satisfy_depends(apt_pkg.parse_depends(d, False)))
 
+
 if __name__ == "__main__":
     _test()
