@@ -35,7 +35,7 @@ either a size or a list containing a size and the numerical value for a
 partition type. The size for partitions is specified in **percentage** of disk
 space, not in bytes (e.g. a size of 33 would take up 1/3 of the disk space).
 The ``overwrite`` option controls whether this module tries to be safe about
-writing partition tables or not. If ``overwrite: false`` is set, the device
+writing partition talbes or not. If ``overwrite: false`` is set, the device
 will be checked for a partition table and for a file system and if either is
 found, the operation will be skipped. If ``overwrite: true`` is set, no checks
 will be performed.
@@ -99,7 +99,6 @@ specified using ``filesystem``.
 
 from cloudinit.settings import PER_INSTANCE
 from cloudinit import util
-from cloudinit import subp
 import logging
 import os
 import shlex
@@ -107,13 +106,13 @@ import shlex
 frequency = PER_INSTANCE
 
 # Define the commands to use
-UDEVADM_CMD = subp.which('udevadm')
-SFDISK_CMD = subp.which("sfdisk")
-SGDISK_CMD = subp.which("sgdisk")
-LSBLK_CMD = subp.which("lsblk")
-BLKID_CMD = subp.which("blkid")
-BLKDEV_CMD = subp.which("blockdev")
-WIPEFS_CMD = subp.which("wipefs")
+UDEVADM_CMD = util.which('udevadm')
+SFDISK_CMD = util.which("sfdisk")
+SGDISK_CMD = util.which("sgdisk")
+LSBLK_CMD = util.which("lsblk")
+BLKID_CMD = util.which("blkid")
+BLKDEV_CMD = util.which("blockdev")
+WIPEFS_CMD = util.which("wipefs")
 
 LANG_C_ENV = {'LANG': 'C'}
 
@@ -164,7 +163,7 @@ def handle(_name, cfg, cloud, log, _args):
 def update_disk_setup_devices(disk_setup, tformer):
     # update 'disk_setup' dictionary anywhere were a device may occur
     # update it with the response from 'tformer'
-    for origname in list(disk_setup):
+    for origname in disk_setup.keys():
         transformed = tformer(origname)
         if transformed is None or transformed == origname:
             continue
@@ -249,11 +248,9 @@ def enumerate_disk(device, nodeps=False):
 
     info = None
     try:
-        info, _err = subp.subp(lsblk_cmd)
+        info, _err = util.subp(lsblk_cmd)
     except Exception as e:
-        raise Exception(
-            "Failed during disk check for %s\n%s" % (device, e)
-        ) from e
+        raise Exception("Failed during disk check for %s\n%s" % (device, e))
 
     parts = [x for x in (info.strip()).splitlines() if len(x.split()) > 0]
 
@@ -313,11 +310,9 @@ def check_fs(device):
 
     blkid_cmd = [BLKID_CMD, '-c', '/dev/null', device]
     try:
-        out, _err = subp.subp(blkid_cmd, rcs=[0, 2])
+        out, _err = util.subp(blkid_cmd, rcs=[0, 2])
     except Exception as e:
-        raise Exception(
-            "Failed during disk check for %s\n%s" % (device, e)
-        ) from e
+        raise Exception("Failed during disk check for %s\n%s" % (device, e))
 
     if out:
         if len(out.splitlines()) == 1:
@@ -432,16 +427,16 @@ def get_dyn_func(*args):
         else:
             return globals()[func_name]
 
-    except KeyError as e:
-        raise Exception("No such function %s to call!" % func_name) from e
+    except KeyError:
+        raise Exception("No such function %s to call!" % func_name)
 
 
 def get_hdd_size(device):
     try:
-        size_in_bytes, _ = subp.subp([BLKDEV_CMD, '--getsize64', device])
-        sector_size, _ = subp.subp([BLKDEV_CMD, '--getss', device])
+        size_in_bytes, _ = util.subp([BLKDEV_CMD, '--getsize64', device])
+        sector_size, _ = util.subp([BLKDEV_CMD, '--getss', device])
     except Exception as e:
-        raise Exception("Failed to get %s size\n%s" % (device, e)) from e
+        raise Exception("Failed to get %s size\n%s" % (device, e))
 
     return int(size_in_bytes) / int(sector_size)
 
@@ -457,11 +452,10 @@ def check_partition_mbr_layout(device, layout):
     read_parttbl(device)
     prt_cmd = [SFDISK_CMD, "-l", device]
     try:
-        out, _err = subp.subp(prt_cmd, data="%s\n" % layout)
+        out, _err = util.subp(prt_cmd, data="%s\n" % layout)
     except Exception as e:
-        raise Exception(
-            "Error running partition command on %s\n%s" % (device, e)
-        ) from e
+        raise Exception("Error running partition command on %s\n%s" % (
+                        device, e))
 
     found_layout = []
     for line in out.splitlines():
@@ -488,11 +482,10 @@ def check_partition_mbr_layout(device, layout):
 def check_partition_gpt_layout(device, layout):
     prt_cmd = [SGDISK_CMD, '-p', device]
     try:
-        out, _err = subp.subp(prt_cmd, update_env=LANG_C_ENV)
+        out, _err = util.subp(prt_cmd, update_env=LANG_C_ENV)
     except Exception as e:
-        raise Exception(
-            "Error running partition command on %s\n%s" % (device, e)
-        ) from e
+        raise Exception("Error running partition command on %s\n%s" % (
+                        device, e))
 
     out_lines = iter(out.splitlines())
     # Skip header.  Output looks like:
@@ -662,11 +655,9 @@ def purge_disk(device):
             wipefs_cmd = [WIPEFS_CMD, "--all", "/dev/%s" % d['name']]
             try:
                 LOG.info("Purging filesystem on /dev/%s", d['name'])
-                subp.subp(wipefs_cmd)
-            except Exception as e:
-                raise Exception(
-                    "Failed FS purge of /dev/%s" % d['name']
-                ) from e
+                util.subp(wipefs_cmd)
+            except Exception:
+                raise Exception("Failed FS purge of /dev/%s" % d['name'])
 
     purge_disk_ptable(device)
 
@@ -691,7 +682,7 @@ def read_parttbl(device):
     blkdev_cmd = [BLKDEV_CMD, '--rereadpt', device]
     util.udevadm_settle()
     try:
-        subp.subp(blkdev_cmd)
+        util.subp(blkdev_cmd)
     except Exception as e:
         util.logexc(LOG, "Failed reading the partition table %s" % e)
 
@@ -706,27 +697,25 @@ def exec_mkpart_mbr(device, layout):
     # Create the partitions
     prt_cmd = [SFDISK_CMD, "--Linux", "--unit=S", "--force", device]
     try:
-        subp.subp(prt_cmd, data="%s\n" % layout)
+        util.subp(prt_cmd, data="%s\n" % layout)
     except Exception as e:
-        raise Exception(
-            "Failed to partition device %s\n%s" % (device, e)
-        ) from e
+        raise Exception("Failed to partition device %s\n%s" % (device, e))
 
     read_parttbl(device)
 
 
 def exec_mkpart_gpt(device, layout):
     try:
-        subp.subp([SGDISK_CMD, '-Z', device])
+        util.subp([SGDISK_CMD, '-Z', device])
         for index, (partition_type, (start, end)) in enumerate(layout):
             index += 1
-            subp.subp([SGDISK_CMD,
+            util.subp([SGDISK_CMD,
                        '-n', '{}:{}:{}'.format(index, start, end), device])
             if partition_type is not None:
                 # convert to a 4 char (or more) string right padded with 0
                 # 82 -> 8200.  'Linux' -> 'Linux'
                 pinput = str(partition_type).ljust(4, "0")
-                subp.subp(
+                util.subp(
                     [SGDISK_CMD, '-t', '{}:{}'.format(index, pinput), device])
     except Exception:
         LOG.warning("Failed to partition device %s", device)
@@ -836,7 +825,6 @@ def lookup_force_flag(fs):
         'btrfs': '-f',
         'xfs': '-f',
         'reiserfs': '-f',
-        'swap': '-f',
     }
 
     if 'ext' in fs.lower():
@@ -978,9 +966,9 @@ def mkfs(fs_cfg):
                 fs_cmd)
     else:
         # Find the mkfs command
-        mkfs_cmd = subp.which("mkfs.%s" % fs_type)
+        mkfs_cmd = util.which("mkfs.%s" % fs_type)
         if not mkfs_cmd:
-            mkfs_cmd = subp.which("mk%s" % fs_type)
+            mkfs_cmd = util.which("mk%s" % fs_type)
 
         if not mkfs_cmd:
             LOG.warning("Cannot create fstype '%s'.  No mkfs.%s command",
@@ -1005,8 +993,8 @@ def mkfs(fs_cfg):
     LOG.debug("Creating file system %s on %s", label, device)
     LOG.debug("     Using cmd: %s", str(fs_cmd))
     try:
-        subp.subp(fs_cmd, shell=shell)
+        util.subp(fs_cmd, shell=shell)
     except Exception as e:
-        raise Exception("Failed to exec of '%s':\n%s" % (fs_cmd, e)) from e
+        raise Exception("Failed to exec of '%s':\n%s" % (fs_cmd, e))
 
 # vi: ts=4 expandtab
