@@ -396,16 +396,16 @@ class Distro(object):
         else:
             create_groups = True
 
-        useradd_cmd = ['useradd', name]
-        log_useradd_cmd = ['useradd', name]
+        adduser_cmd = ['useradd', name]
+        log_adduser_cmd = ['useradd', name]
         if util.system_is_snappy():
-            useradd_cmd.append('--extrausers')
-            log_useradd_cmd.append('--extrausers')
+            adduser_cmd.append('--extrausers')
+            log_adduser_cmd.append('--extrausers')
 
         # Since we are creating users, we want to carefully validate the
         # inputs. If something goes wrong, we can end up with a system
         # that nobody can login to.
-        useradd_opts = {
+        adduser_opts = {
             "gecos": '--comment',
             "homedir": '--home',
             "primary_group": '--gid',
@@ -418,7 +418,7 @@ class Distro(object):
             "selinux_user": '--selinux-user',
         }
 
-        useradd_flags = {
+        adduser_flags = {
             "no_user_group": '--no-user-group',
             "system": '--system',
             "no_log_init": '--no-log-init',
@@ -453,32 +453,32 @@ class Distro(object):
         # Check the values and create the command
         for key, val in sorted(kwargs.items()):
 
-            if key in useradd_opts and val and isinstance(val, str):
-                useradd_cmd.extend([useradd_opts[key], val])
+            if key in adduser_opts and val and isinstance(val, str):
+                adduser_cmd.extend([adduser_opts[key], val])
 
                 # Redact certain fields from the logs
                 if key in redact_opts:
-                    log_useradd_cmd.extend([useradd_opts[key], 'REDACTED'])
+                    log_adduser_cmd.extend([adduser_opts[key], 'REDACTED'])
                 else:
-                    log_useradd_cmd.extend([useradd_opts[key], val])
+                    log_adduser_cmd.extend([adduser_opts[key], val])
 
-            elif key in useradd_flags and val:
-                useradd_cmd.append(useradd_flags[key])
-                log_useradd_cmd.append(useradd_flags[key])
+            elif key in adduser_flags and val:
+                adduser_cmd.append(adduser_flags[key])
+                log_adduser_cmd.append(adduser_flags[key])
 
         # Don't create the home directory if directed so or if the user is a
         # system user
         if kwargs.get('no_create_home') or kwargs.get('system'):
-            useradd_cmd.append('-M')
-            log_useradd_cmd.append('-M')
+            adduser_cmd.append('-M')
+            log_adduser_cmd.append('-M')
         else:
-            useradd_cmd.append('-m')
-            log_useradd_cmd.append('-m')
+            adduser_cmd.append('-m')
+            log_adduser_cmd.append('-m')
 
         # Run the command
         LOG.debug("Adding user %s", name)
         try:
-            util.subp(useradd_cmd, logstring=log_useradd_cmd)
+            util.subp(adduser_cmd, logstring=log_adduser_cmd)
         except Exception as e:
             util.logexc(LOG, "Failed to create user %s", name)
             raise e
@@ -490,15 +490,15 @@ class Distro(object):
 
         snapuser = kwargs.get('snapuser')
         known = kwargs.get('known', False)
-        create_user_cmd = ["snap", "create-user", "--sudoer", "--json"]
+        adduser_cmd = ["snap", "create-user", "--sudoer", "--json"]
         if known:
-            create_user_cmd.append("--known")
-        create_user_cmd.append(snapuser)
+            adduser_cmd.append("--known")
+        adduser_cmd.append(snapuser)
 
         # Run the command
         LOG.debug("Adding snap user %s", name)
         try:
-            (out, err) = util.subp(create_user_cmd, logstring=create_user_cmd,
+            (out, err) = util.subp(adduser_cmd, logstring=adduser_cmd,
                                    capture=True)
             LOG.debug("snap create-user returned: %s:%s", out, err)
             jobj = util.load_json(out)
@@ -577,27 +577,15 @@ class Distro(object):
         """
         Lock the password of a user, i.e., disable password logins
         """
-        # passwd must use short '-l' due to SLES11 lacking long form '--lock'
-        lock_tools = (['passwd', '-l', name], ['usermod', '--lock', name])
         try:
-            cmd = next(l for l in lock_tools if util.which(l[0]))
-        except StopIteration:
-            raise RuntimeError((
-                "Unable to lock user account '%s'. No tools available. "
-                "  Tried: %s.") % (name, [c[0] for c in lock_tools]))
-        try:
-            util.subp(cmd)
+            # Need to use the short option name '-l' instead of '--lock'
+            # (which would be more descriptive) since SLES 11 doesn't know
+            # about long names.
+            util.subp(['passwd', '-l', name])
         except Exception as e:
             util.logexc(LOG, 'Failed to disable password for user %s', name)
             raise e
 
-    def expire_passwd(self, user):
-        try:
-            util.subp(['passwd', '--expire', user])
-        except Exception as e:
-            util.logexc(LOG, "Failed to set 'expire' for %s", user)
-            raise e
-
     def set_passwd(self, user, passwd, hashed=False):
         pass_string = '%s:%s' % (user, passwd)
         cmd = ['chpasswd']
