@@ -15,8 +15,6 @@ import os
 import re
 import time
 
-import six
-
 from cloudinit import log as logging
 from cloudinit import sources
 from cloudinit import util
@@ -148,9 +146,6 @@ class DataSourceOVF(sources.DataSource):
                     product_marker, os.path.join(self.paths.cloud_dir, 'data'))
                 special_customization = product_marker and not hasmarkerfile
                 customscript = self._vmware_cust_conf.custom_script_name
-                ccScriptsDir = os.path.join(
-                    self.paths.get_cpath("scripts"),
-                    "per-instance")
             except Exception as e:
                 _raise_error_status(
                     "Error parsing the customization Config File",
@@ -204,9 +199,7 @@ class DataSourceOVF(sources.DataSource):
 
                 if customscript:
                     try:
-                        postcust = PostCustomScript(customscript,
-                                                    imcdirpath,
-                                                    ccScriptsDir)
+                        postcust = PostCustomScript(customscript, imcdirpath)
                         postcust.execute()
                     except Exception as e:
                         _raise_error_status(
@@ -239,11 +232,11 @@ class DataSourceOVF(sources.DataSource):
                 GuestCustErrorEnum.GUESTCUST_ERROR_SUCCESS)
 
         else:
-            np = [('com.vmware.guestInfo', transport_vmware_guestinfo),
-                  ('iso', transport_iso9660)]
+            np = {'iso': transport_iso9660,
+                  'vmware-guestd': transport_vmware_guestd, }
             name = None
-            for name, transfunc in np:
-                contents = transfunc()
+            for (name, transfunc) in np.items():
+                (contents, _dev, _fname) = transfunc()
                 if contents:
                     break
             if contents:
@@ -282,12 +275,6 @@ class DataSourceOVF(sources.DataSource):
         self.cfg = cfg
         return True
 
-    def _get_subplatform(self):
-        system_type = util.read_dmi_data("system-product-name").lower()
-        if system_type == 'vmware':
-            return 'vmware (%s)' % self.seed
-        return 'ovf (%s)' % self.seed
-
     def get_public_ssh_keys(self):
         if 'public-keys' not in self.metadata:
             return []
@@ -441,7 +428,7 @@ def maybe_cdrom_device(devname):
     """
     if not devname:
         return False
-    elif not isinstance(devname, six.string_types):
+    elif not isinstance(devname, util.string_types):
         raise ValueError("Unexpected input for devname: %s" % devname)
 
     # resolve '..' and multi '/' elements
@@ -471,8 +458,8 @@ def maybe_cdrom_device(devname):
     return cdmatch.match(devname) is not None
 
 
-# Transport functions are called with no arguments and return
-# either None (indicating not present) or string content of an ovf-env.xml
+# Transport functions take no input and return
+# a 3 tuple of content, path, filename
 def transport_iso9660(require_iso=True):
 
     # Go through mounts to see if it was already mounted
@@ -484,9 +471,9 @@ def transport_iso9660(require_iso=True):
         if not maybe_cdrom_device(dev):
             continue
         mp = info['mountpoint']
-        (_fname, contents) = get_ovf_env(mp)
+        (fname, contents) = get_ovf_env(mp)
         if contents is not False:
-            return contents
+            return (contents, dev, fname)
 
     if require_iso:
         mtype = "iso9660"
@@ -499,33 +486,29 @@ def transport_iso9660(require_iso=True):
             if maybe_cdrom_device(dev)]
     for dev in devs:
         try:
-            (_fname, contents) = util.mount_cb(dev, get_ovf_env, mtype=mtype)
+            (fname, contents) = util.mount_cb(dev, get_ovf_env, mtype=mtype)
         except util.MountFailedError:
             LOG.debug("%s not mountable as iso9660", dev)
             continue
 
         if contents is not False:
-            return contents
+            return (contents, dev, fname)
 
-    return None
+    return (False, None, None)
 
 
-def transport_vmware_guestinfo():
-    rpctool = "vmware-rpctool"
-    not_found = None
-    if not util.which(rpctool):
-        return not_found
-    cmd = [rpctool, "info-get guestinfo.ovfEnv"]
-    try:
-        out, _err = util.subp(cmd)
-        if out:
-            return out
-        LOG.debug("cmd %s exited 0 with empty stdout: %s", cmd, out)
-    except util.ProcessExecutionError as e:
-        if e.exit_code != 1:
-            LOG.warning("%s exited with code %d", rpctool, e.exit_code)
-            LOG.debug(e)
-    return not_found
+def transport_vmware_guestd():
+    # http://blogs.vmware.com/vapp/2009/07/ \
+    #    selfconfiguration-and-the-ovf-environment.html
+    # try:
+    #     cmd = ['vmware-guestd', '--cmd', 'info-get guestinfo.ovfEnv']
+    #     (out, err) = subp(cmd)
+    #     return(out, 'guestinfo.ovfEnv', 'vmware-guestd')
+    # except:
+    #     # would need to error check here and see why this failed
+    #     # to know if log/error should be raised
+    #     return(False, None, None)
+    return (False, None, None)
 
 
 def find_child(node, filter_func):
