@@ -4,21 +4,22 @@
 #
 # This file is part of cloud-init. See LICENSE file for license information.
 
-__VERSION__ = "19.4"
-_PACKAGED_VERSION = '19.4-33-gbb4131a2-0ubuntu1~16.04.1'
+__VERSION__ = "22.2"
+_PACKAGED_VERSION = "22.2-0ubuntu1~18.04.2"
 
 FEATURES = [
     # supports network config version 1
-    'NETWORK_CONFIG_V1',
+    "NETWORK_CONFIG_V1",
     # supports network config version 2 (netplan)
-    'NETWORK_CONFIG_V2',
+    "NETWORK_CONFIG_V2",
 ]
 
 
 def version_string():
     """Extract a version string from cloud-init."""
-    if not _PACKAGED_VERSION.startswith('@@'):
+    if not _PACKAGED_VERSION.startswith("@@"):
         return _PACKAGED_VERSION
     return __VERSION__
 
+
 # vi: ts=4 expandtab
